/*
 * Accelerometer.c
 *
 *
 *      Author: Cesar López
 *		Last Update:03/01/2017
 */

#include "LPC17xx.h"
#include "log.h"
#include "serial.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"
#include "config.h"
#include "type.h"
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "stdbool.h"
#include "accelerometer.h"
#include "math.h"
#include "AlMonitor.h"
#include "led.h"
#include "adc.h"
#include "gpio.h"


static portTickType pxFirstAlarm = -1;
static portTickType pxActivity = -1;

static int8_t iKneels=0;

static float fActX=0;
static float fActY=0;
static float fActZ=0;

static bool bWatchON = false;
static bool bAlarmSended = false;
static bool bActiSended = false;

/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */

static void _vSendToMonitor(uint8_t iSender, bool bActive,  uint8_t iPar,   uint16_t iVal);
static bool _bCheckAngle(sReadAccel *sChData);


/** Configure alarm or signal and send it to monitor queue
 *
 *  @param sRData data structure with raw data from ADC(sended from APP main)
 *
 *
 *  @return void
 */


static void _vSendToMonitor(uint8_t iSender, bool bActive,  uint8_t iPar,   uint16_t iVal)
{
	sSendAlarm xAlarmFuncAcc;

	xAlarmFuncAcc.iOPT=iSender;
	xAlarmFuncAcc.iTry=0;
	xAlarmFuncAcc.bIsON=bActive;

	if (iSender < 99)
	{
		xAlarmFuncAcc.iValue = 0;
		xAlarmFuncAcc.iParam = 0;
	}
	else
	{
		xAlarmFuncAcc.iValue = iVal;
		xAlarmFuncAcc.iParam = iPar;
	}

	vAlMonitor_AddAlarm(&xAlarmFuncAcc,false);

	return;
}


/** Check if angles are good or limits has been  exceded
 *
 *  @param sChData data structure with raw data from ADC and G-forces calculated previously
 *
 *
 *  @return bool true if angles are inverted
 */


static bool _bCheckAngle(sReadAccel *sChData)
{
	/*Normal situation is
	 *
	 *  70 º> Angle_Z > 130º
	 * Angle_X > 70   (large edge)
	 *
	 */

	if ((sChData->faZ>140)||(sChData->faZ<70) || (sChData->faX<60)|| (sChData->faX>120))
	{
		if (pxFirstAlarm==-1)
		{
			bWatchON=true;
			pxFirstAlarm = xTaskGetTickCount();  //If it is the first one save tick count

		}
	}
	else
	{
		/* Reset alarm State*/
		if (pxFirstAlarm!=-1)
		{
			bWatchON=false;
			pxFirstAlarm=-1;
		}
	}
	return bWatchON;
}


/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Set GPIO and select resolution for accelerometer
 *
 *  @param void
 *
 *  @return void
 *
 */


void Accel_vInit(void)
{

	Gpio_config(18, GPIO_OUT );

	//Low resolution mode
	Gpio_set(18,0);


    return;
}


/** Calculate G-Forces and angles from sRData raw reads.
 *
 *  @param sRData data structure with raw data from ADC(sended from APP main)
 *
 *
 *  @return void
 */


void Accel_sRAccel ( sReadAccel *sRData )
{
	float fModule=0;
	static bool iKneelCounted=false;
	static portTickType pxBeginMonitor = -1;
	static int16_t iAvAccel=0;

	if (pxBeginMonitor==-1) pxBeginMonitor = xTaskGetTickCount();

	//ADCReadAcc( sRData);

	sRData->fpX = (((sRData->fX * 3.3) / 4096) - 1.65) / 0.800;
	sRData->fpY  = (((sRData->fY * 3.3) / 4096) - 1.65) / 0.800;
	sRData->fpZ  = (((sRData->fZ* 3.3) / 4096) - 1.65) / 0.800;
	fModule = sqrtf( powf(sRData->fpX,2) + powf(sRData->fpY,2) + powf(sRData->fpZ,2));

	sRData->faX = (180*acosf(sRData->fpX/fModule))/PI;
	sRData->faY = (180*acosf(sRData->fpY/fModule))/PI;
	sRData->faZ = (180*acosf(sRData->fpZ/fModule))/PI;

	Log_print(mACCEL,"Received in ACCELEROMETER(G,Angle):    X :  (%f,%f)\tY: (%f,%f)\tZ: (%f,%f)\r\n",sRData->fpX,sRData->faX,sRData->fpY,sRData->faY,sRData->fpZ,sRData->faZ);

	/*
	 * Add values to iAvAccel as Data from mACCEL, only if person has not fallen.(Gz is good for iAvAccel)
	 */
	if (bAlarmSended == false)
	{
		iAvAccel = iAvAccel + abs((int16_t)(sRData->fpZ*100)) + abs((int16_t)(sRData->fpX*100));
		Log_print(mACCEL,"mACCEL says: value of iAvAccel is %d\r\n",iAvAccel);
	}
	/*Check Activity*/

	if (fActX==0 && fActY==0 && fActZ==0)
	{
		fActX = sRData->fpX;
		fActY = sRData->fpY;
		fActZ = sRData->fpZ;
		pxActivity = xTaskGetTickCount();	}
	else
	{
		if (!( fabs(fabs(fActZ) - fabs(sRData->fpZ)) <0.06 && fabs(fabs(fActZ) - fabs(sRData->fpZ)) < 0.06 && fabs(fabs(fActZ) - fabs(sRData->fpZ))<0.06))
		{
			if (bActiSended == true)
			{
				_vSendToMonitor(mFUNC_ACACT,false,0,0);
				bActiSended = false;
			}
		fActX = sRData->fpX;
		fActY = sRData->fpY;
		fActZ = sRData->fpZ;
		pxActivity = xTaskGetTickCount();		}
		if ((xTaskGetTickCount()-pxActivity) >= mMILIS_INAC/portTICK_RATE_MS)
		{
			if (bActiSended == false )
			{
				Log_print(mACCEL,"mACCEL says: INACTIVITY ALARM ACTIVATED, SENDING MESSAGE\r\n");
				_vSendToMonitor(mFUNC_ACACT,true,0,0);
				bActiSended =true;
			}
		}
	}



	/*Check ANGLES*/

	if (_bCheckAngle(sRData)==true)
	{
		if (iKneelCounted==false)
			{
				iKneels++;
				iKneelCounted=true;
			}
		if ((xTaskGetTickCount()-pxFirstAlarm) >= mMILIS_FALLING/portTICK_RATE_MS)
		{
			if (bAlarmSended==false)
			{
				Log_print(mACCEL,"mACCEL says: FALL ALARM ACTIVATED, SENDING MESSAGE\r\n");
				bAlarmSended=true;
				_vSendToMonitor(mFUNC_ACCEL,true,0,0);
			}
		}
	}
	else
		{
			iKneelCounted=false;
		 	if (bAlarmSended==true)
		 		{
		 			/*Readings are good again*/
					Log_print(mACCEL,"mACCEL says: FALL ALARM DEACTIVATED, SENDING MESSAGE\r\n");
					bAlarmSended=false;
					_vSendToMonitor(mFUNC_ACCEL,false,0,0);

	 		}
		}

	if ((xTaskGetTickCount()-pxBeginMonitor > mTIME_MONIT/portTICK_RATE_MS) )
	{
		pxBeginMonitor = xTaskGetTickCount();
		if (iKneels>0)
		{
			_vSendToMonitor(mDATA_ACCEL,true,1,iKneels);
			iKneels=0;
		}
		if (iAvAccel>0)
		{
			_vSendToMonitor(mDATA_ACCEL,true,2,iAvAccel);
			iAvAccel = 0;
		}
	}
	return;
}

