/*
 * Thermometer.c
 *
 *  Created on: 25 de dic. de 2016
 *      Author: Cesar_root
 */


#include "LPC17xx.h"
#include "log.h"
#include "FreeRTOS.h"
#include "task.h"
#include "config.h"
#include "stdbool.h"
#include "stdlib.h"
#include "Thermometer.h"
#include "math.h"
#include <stdlib.h>


static int8_t iLimits[mNUM_LIMITS];
static int8_t iZone=0;

/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */

/** Transform the value readed from ADC in mV*100 integer
 *
 * @param iADC Value read from ADC
 *
 * @return value integer mV*100
 */

static int8_t _ADCToVoltx100(uint16_t iADC)
{
	return  (int8_t)(10*(((iADC*33) >> 7))>> 5);
}

/** Transform the value  mV*100 integer in temperature in ºC*100
 *
 * @param iVolts returned from _ADCToVoltx100
 *
 * @return temperature in ºC*100
 */

static int16_t _VoltToTemp(int8_t iVolts)
{
	return (int16_t)((iVolts*104)-5400);
}

/** Check in what temperature zone defined in Header is the system
 *
 * @param iVolTemp Temperature read(returned from _ADCToVoltx100)
 *
 * @return Zone obtained
 */

int8_t _ChkZone(int8_t iVolTemp)
{
	static int8_t iIndex;
	Log_print(mTHERM,"mTHERM says: Received mV*100 of ADC reading: %d\r\n",iVolTemp);
	for( iIndex = 0 ; iIndex < mNUM_LIMITS ; iIndex++ )
	{
		if (iVolTemp < iLimits[iIndex]) return iIndex;
	}

	return mNUM_LIMITS;
}

/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Return temperature from ADC read
 *
 * @param iADC value read from ADC
 *
 * @return temperature in ºC*100
 */

int16_t Thermo_GetTemp(int16_t iADC)
{
	return _VoltToTemp(_ADCToVoltx100(iADC));
}

/** Load temperature zone limits defined in header
 *
 * @param void
 *
 * @return void
 */

void Thermo_SetUp(void)
{

	iLimits[0] = (int8_t)mTEMP_LIM1;
	iLimits[1] = (int8_t)mTEMP_LIM2;
	iLimits[2] = (int8_t)mTEMP_LIM3;
	iLimits[3] = (int8_t)mTEMP_LIM4;
	iLimits[4] = (int8_t)mTEMP_LIM5;
	iLimits[5] = (int8_t)mTEMP_LIM6;

	Log_print(mGENER,"mTHERM says: THERM has initated!!\r\n");
}

/** Get Method
 *
 * @param void
 *
 * @return Temperature zone
 */

int8_t Thermo_GetZone(void)
{
	return iZone;
}

/** Check if zone has changed after new reading of temperature
 *
 * @param iADC read
 *
 * @return bool true if zone has changed
 */

bool Thermo_ChkTemp(int16_t iADC)
{
	static int8_t bSended = -1;
	static portTickType pTime=-1;
	static int8_t iNewZone = 0;



	iNewZone = _ChkZone(_ADCToVoltx100(iADC));
	if ((pTime == -1)  || (iZone != iNewZone))
	{
		iZone = iNewZone;
		pTime = xTaskGetTickCount();
		Log_print(mTHERM,"mTHERM says: THERMIC ZONE has changed to %d\r\n",iZone);
		return false;
	}

		//check time of exposition
		if (bSended != iZone)
		{
			if (xTaskGetTickCount()-pTime >= mTEMP_THRES)
			{
				bSended = iZone;
				return true;
			}
		}

return false;
}
/** Permanent monitor of temperature according to resolution defined in header(mTEMP_RESOL)
 *
 * @param iADC read
 *
 * @return bool true mTEMP_RESOL has been exceded
 */

bool Thermo_ChkTMonitor(int16_t iADC)
{
	static int8_t iTemp1 = -1;
	static portTickType pTime=-1;

	if (iTemp1 == -1)
	{
		pTime = xTaskGetTickCount();
		iTemp1 = _ADCToVoltx100(iADC);
		return true;
	}
	else
	{
		if ((abs(iTemp1-_ADCToVoltx100(iADC)) > mTEMP_RESOL ) && ((xTaskGetTickCount()-pTime)>2700/portTICK_RATE_MS))
		{
			pTime = xTaskGetTickCount();
			iTemp1 = _ADCToVoltx100(iADC);
			return true;
		}
	}
return false;
}
