/*
 * adc.c
 *
 *
 *      Author: Cesar López
 *		Last Update: 03/01/2017
 */

#include "LPC17xx.h"
#include "log.h"
#include "serial.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "config.h"
#include "math.h"
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "stdbool.h"
#include "led.h"
#include "adc.h"
#include "gpio.h"


xSemaphoreHandle ADCMux;


/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Prepare ADC. Set PCONP and configure operation mode in ADCR
 *
 *  @param uint32_t Clk
 *  @param int iPos: position in the array of the current alarm
 *
 */


void ADC_vADCInit( uint32_t Clk )
{
	 uint32_t clk1, clk2;

	  /* Enable CLOCK into ADC controller so it is ON(bit 12 PCADC
	   * ADC now is working
	   */

		if (ADCMux == NULL) ADCMux = xSemaphoreCreateRecursiveMutex();
		xSemaphoreTake( ADCMux, portMAX_DELAY );

	  LPC_SC->PCONP |= (1 << 12);
	  /* Set pins for accelerometer  pinsel1 mode 01*/
	  LPC_PINCON->PINSEL1 &= ~0x003FC000;
	  LPC_PINCON->PINSEL1 |= 0x00154000;

	  LPC_PINCON->PINSEL3 |= 0xF0000000;


	  clk1 = (LPC_SC->PCLKSEL0 >> 6) & 0x03;
	  switch ( clk1 )
	  {
		case 0x00:
		default:
		  clk2 = SystemCoreClock/4;
		break;
		case 0x01:
		  clk2 = SystemCoreClock;
		break;
		case 0x02:
		  clk2 = SystemCoreClock/2;
		break;
		case 0x03:
		  clk2 = SystemCoreClock/8;
		break;
	  }
	  /*Configure ADCR to make ADC work properly and
	   * by default set channel 0 to read
	   * start (bit 24 is down until adc is needed to work)
	   */

	  LPC_ADC->ADCR = ( 0x01 << 0 ) |( ( clk2  / Clk - 1 ) << 8 ) |  ( 0 << 16 ) | ( 0 << 17 ) | ( 1 << 21 ) | 	( 0 << 24 ) |  	( 0 << 27 );

	  xSemaphoreGive( ADCMux );

	  return;
}

/** Read the three lower channels ADC (accelerometer)
 *
 *  @param sReadAccel *sAData : Structure for the data read
 *
 *  @return void
 */

void ADC_ADCAverage(sReadAccel *sAData )
{

	sAData->fX=0;
	sAData->fY=0;
	sAData->fZ=0;

	for (int iCount=0;iCount<300;iCount++)
	{
		sAData->fX = sAData->fX + ADC_Read(0);
		sAData->fY = sAData->fY + ADC_Read(1);
		sAData->fZ = sAData->fZ + ADC_Read(2);

	}

	sAData->fX=	sAData->fX/300;
	sAData->fY=	sAData->fY/300;
	sAData->fZ=	sAData->fZ/300;

	Log_print(mANALO,"Readings in ADC(1000 Readings):    C0 :  %i     C1: %i     C2: %i \r\n",sAData->fX,sAData->fY,sAData->fZ);
    return;
}


/** Read the selected channel from ADC
 *
 * @param void
 *
 * @return value read
 */


uint32_t  ADC_Read( uint8_t ADCChan )
{
	uint32_t  iValue,iDataRet;


	xSemaphoreTake( ADCMux, portMAX_DELAY );


	// Reset channel selector and control bits 24..26
  LPC_ADC->ADCR &= 0xF8FFFF00;
  // Select channel to listen to and convert and start conversion (bit 24 high)
  LPC_ADC->ADCR |= (1 << 24) | (1 << ADCChan);
  while ( 1 )
  {
		switch ( ADCChan )
		{
		  case 0:
			  iValue = LPC_ADC->ADDR0;
		  break;
		  case 1:
			  iValue = LPC_ADC->ADDR1;
		  break;
		  case 2:
			  iValue = LPC_ADC->ADDR2;
		  break;
		  case 3:
			  iValue = LPC_ADC->ADDR3;
		  break;
		  case 4:
			  iValue = LPC_ADC->ADDR4;
		  break;
		}
		/*
		 * Check if it's done and there is data converted
		 */
		if ( iValue & 0x80000000 )
		{
			break;
		}
  }
  LPC_ADC->ADCR &= 0xF8FFFFFF;	/* stop ADC now */
  /*
   * Shift to the right 4 bits reserved and take 12 bits of data with the mask 0xFFF
   */
  iDataRet = (( iValue >> 4 ) & 0xFFF);

  xSemaphoreGive( ADCMux);

  return (iDataRet);/* return A/D conversion value */

}
