/*
 * log.c
 *
 *  Created on: Nov 9, 2016
 *      Author: jordi
 *
 *
 *  Modified by : César López
 *  Last Update 31/12/2016
 */
#include "LPC17xx.h"
#include "log.h"
#include "serial.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "config.h"
#include <stdarg.h>
#include "UartScanf.h"
#include <stdlib.h>
#include <string.h>
#include "stdbool.h"
#include "led.h"
#include "wifly.h"



/* Declare a variable of type xSemaphoreHandle */

volatile static xSemaphoreHandle PrintMutex = NULL;

/*Declare a variable to show the line number in the next log print*/

static uint8_t LogLine = 0;

//Declare and init log filters array.(All is shown as default)

static bool bLogFilters[mMAXFI];


uint_fast8_t log_port = BAUD_UNSUPORTED;	// serial port used to send log

/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */

/**
 *
 * Manage filters to decide wether to show or not a message
 * @param uint_fast8_t iFilterID
 *
 * @return void
 */

static void _vSetFilter(uint_fast8_t iFilterID)
{
	uint_fast8_t iIndex;

	if (iFilterID == mMAXFI)
	{
		/*Reset filters*/
		bLogFilters[0]=true;
		Log_print(mGENER,"LOG says: Mode verbose has been RESETED. Only mGENER is UP!\r\n");

		for (iIndex=1;iIndex<=mMAXFI-1;iIndex++)
		{
			bLogFilters[iIndex]=false;
		}
 	}
	else
	{
		if (bLogFilters[iFilterID]==false)
		{
			bLogFilters[iFilterID]=true;
			Log_print(mGENER,"LOG says: Mode verbose has been setted UP for %i\r\n",iFilterID);
		}
		else
		{
			Log_print(mGENER,"LOG says: Mode verbose has been setted DOWN for %i\r\n",iFilterID);
			bLogFilters[iFilterID]=false;
		}
	}
}

/**
 * Manage mutex for print in log
 *
 *@param void
 *
 *@return void
 */

static void _vLockLogPrint()
{

	if(PrintMutex == NULL)
	{
		PrintMutex = xSemaphoreCreateMutex();
	}

	xSemaphoreTake(PrintMutex, portMAX_DELAY);
}

/**
 * Manage mutex for print in log
 *
 *@param void
 *
 *@return void
 */

static void _vUnlockLogPrint()
{
	xSemaphoreGive(PrintMutex);
}


/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Read a user command from mUART_LOG and do it
 *
 * @param void
 *
 * @return void
 */

void Log_vReadLogCmd()
{
	char cLogMenuOpc;
	uint_fast8_t iNumberChars;

	iNumberChars=UartScanf_iScanSerial(mUART_LOG,100,&cLogMenuOpc);

	if (iNumberChars!=0)
	{

		if((cLogMenuOpc=='a')||(cLogMenuOpc=='A'))
		{
			_vSetFilter(mACCEL);
		}
		else if((cLogMenuOpc=='p')||(cLogMenuOpc=='P'))
		{
			_vSetFilter(mPULSO);
		}
		else if((cLogMenuOpc=='r')||(cLogMenuOpc=='R'))
		{
			_vSetFilter(mMAXFI);
		}
		else if((cLogMenuOpc=='t')||(cLogMenuOpc=='T'))
		{
			_vSetFilter(mTHERM);
		}
		else if((cLogMenuOpc=='d')||(cLogMenuOpc=='D'))
				{
					_vSetFilter(mANALO);
				}
		else if((cLogMenuOpc=='c')||(cLogMenuOpc=='C'))
				{
					_vSetFilter(mCOMMU);
				}
		else
		{
			Log_vPrintMenu();
		}
	}
}



/** Init LOG vars and init Filters
 *
 * @param uint_fast8_t port   port (typically mUART_LOG)
 * @param serial_baudrate baudRate
 *
 */

uint_fast8_t Log_start( uint_fast8_t port, serial_baudrate baudRate ) {
	uint_fast8_t iSerialStart;
	log_port = port;
	iSerialStart=Serial_start(port, baudRate);
	_vSetFilter(mMAXFI);
	return iSerialStart;
}

/*
 * Print log text
 *
 * @param iFilter Filter to check if text has to be displayed or not
 * @param format 	like printf of cstdio C standard
 *
 * @return len of buffer_tx
 */

uint_fast16_t Log_print(uint_fast8_t iFilter,char* format, ...) {

	uint_fast16_t len;

	if (log_port == BAUD_UNSUPORTED) return 0;

	// If the sender is blocked by log filter then do not show message

	if (bLogFilters[iFilter] == false) return 0;

	va_list args;
	va_start( args, format );

	_vLockLogPrint();
	Serial_send(log_port, "LOG Line: %i\t", LogLine);
	len = Serial_vsend(log_port, format, args);
	_vUnlockLogPrint();

	va_end(args);

	LogLine++;

	return len;
}


/** Print menu options in screen
 *
 * @param void
 * @param void
 *
 * @return void
 */

void Log_vPrintMenu()
{
  Log_print(mGENER,"\r\n");
  Log_print(mGENER,"\r\n");
  Log_print(mGENER,"\r\n");
  Log_print(mGENER,"╔═══════════════════════════╦═══════════════════════════╗\r\n");
  Log_print(mGENER,"║                           ║                           ║\r\n");
  Log_print(mGENER,"║   Reading Filters         ║    Other Options          ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║ ────────────────────      ║  ─────────────────────    ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║     A - Accelerometer   %d ║                           ║░░\r\n",bLogFilters[mACCEL]);
  Log_print(mGENER,"║                           ║      R - Reset filters    ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║     P - Pulsometer      %d ║                           ║░░\r\n",bLogFilters[mPULSO]);
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║                           ║      Any key to show menu ║░░\r\n");
  Log_print(mGENER,"║     T - Thermometer     %d ║                           ║░░\r\n",bLogFilters[mTHERM]);
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║     D - ADC             %d ║                           ║░░\r\n",bLogFilters[mANALO]);
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║     C - Comm            %d ║                           ║░░\r\n",bLogFilters[mCOMMU]);
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"║                           ║                           ║░░\r\n");
  Log_print(mGENER,"╚═══════════════════════════╩═══════════════════════════╝░░\r\n");
  Log_print(mGENER,"   ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░\r\n");
  Log_print(mGENER,"\r\n");
  Log_print(mGENER,"\r\n");
  Log_print(mGENER,"Select option\r\r\n");


}


